Attribute VB_Name = "ProcessCalendar"
Option Explicit

' From Tom Howe (howe@controlcenter.com)
' Copyright 2000; All rights reserved.

Public Sub AddItem(bstrURLItem As String)

    Dim objPersonalAppointment As New CDO.Appointment
    Dim objOfficeAppointment As New CDO.Appointment
    Dim strOfficeCalendarURL As String
    Dim strInitials As String
    
    ' Get the first three characters of the users name to put
    ' in front of the items added to the Office Calendar. The users
    ' name is the mailbox name, not the profile name.
    strInitials = GetInitials(bstrURLItem)

    ' Reference the new Appointment added to the Personal Calendar.
    objPersonalAppointment.DataSource.Open bstrURLItem
    
    ' Do not add Holidays to Office Calendar. Outlook adds holidays to personal
    ' calendars and these should not all be replicated to the Office Calendar.
    ' With 200 users, there would be 200 Christmas entries on December 25.
    If objPersonalAppointment.Keywords(0) = "Holiday" Then
    
        ' This is a holiday, do not add it.
        
    Else
    
        ' If the user chose to save the item to the Office Calendar, the
        ' BusyStatus was set to olBusy(2). If so, create an item on the
        ' Office Calendar.
        If objPersonalAppointment.BusyStatus = "BUSY" Then
    
            ' Build a string to the Office Calendar.
            strOfficeCalendarURL = GetExchangeURLDomainString & "Public Folders/Office Calendar/"
        
            With objOfficeAppointment
        
                ' Save the Message ID in the Office Calendar Appointment.
                .Fields("urn:schemas:mailheader:message-id") = objPersonalAppointment.Fields("urn:schemas:mailheader:message-id")
                .Fields.Update
                
                ' Put the values in the new Appointment item for the Office Calendar.
                .AllDayEvent = objPersonalAppointment.AllDayEvent
                .BusyStatus = objPersonalAppointment.BusyStatus
                .Contact = objPersonalAppointment.Contact
                .ContactURL = objPersonalAppointment.ContactURL
                .EndTime = objPersonalAppointment.EndTime
                .Keywords = objPersonalAppointment.Keywords
                .MeetingStatus = objPersonalAppointment.MeetingStatus
                .Priority = objPersonalAppointment.Priority
                .Resources = objPersonalAppointment.Resources
                .ResponseText = objPersonalAppointment.ResponseText
                .Sensitivity = objPersonalAppointment.Sensitivity
                .StartTime = objPersonalAppointment.StartTime
                
                ' If the Appointment is marked as "Private" change the
                ' subject and body of the appointment to "Private."
                If .Sensitivity = 2 Then
                    .Location = "Private"
                    .LocationURL = vbNullString
                    .Subject = strInitials & "Private"
                    .TextBody = "Private"
                Else
                    .Location = objPersonalAppointment.Location
                    .LocationURL = objPersonalAppointment.LocationURL
                    .Subject = strInitials & objPersonalAppointment.Subject
                    .TextBody = objPersonalAppointment.TextBody
                End If
                
                .Transparent = objPersonalAppointment.Transparent
              
                ' Does not include the collection of attachments since Office Calendar
                ' is used to reference scheduled information only.
        
                ' Save the new Appointment to the Office Calendar.
                .DataSource.SaveToContainer strOfficeCalendarURL
        
            End With
        
        End If
        
    End If

    ' Destroy the object variables.
    Set objPersonalAppointment = Nothing
    Set objOfficeAppointment = Nothing

End Sub

Public Sub DeleteItem(bstrURLItem As String)

        ' This is the first time the event sink has been called. Here, you
        ' can access values from the deleted item.
   
        Dim objPersonalAppointment As New CDO.Appointment
        Dim rst As New ADODB.Recordset
        Dim record As New ADODB.record
        Dim strOfficeCalendarURL As String
        Dim strItem As String
                    
        ' Reference the new Appointment added to the Personal Calendar.
        objPersonalAppointment.DataSource.Open bstrURLItem
          
        ' Check the personal calendar item that is deleted. If the BusyStatus
        ' property equal "Busy" there is an associated Office Calendar
        ' appointment item so delete it.
        If objPersonalAppointment.BusyStatus = "BUSY" Then
          
            ' Build a string to the Office Calendar.
            strOfficeCalendarURL = GetExchangeURLDomainString & "Public Folders/Office Calendar/"
                      
            ' Save the Message ID of the deleted message. This Message ID will be used to
            ' delete the item from the Office Calendar.
            strItem = objPersonalAppointment.Fields("urn:schemas:mailheader:message-id")
    
            'Open a record based on the Office Calendar folder.
            record.Open strOfficeCalendarURL
    
            ' Set the connection to the Office Calendar folder.
            Set rst.ActiveConnection = record.ActiveConnection
            
            With rst
            
                ' Use a SQL statement for the source of the recordset.
                .Source = "SELECT ""DAV:href"", " & _
                              " ""urn:schemas:mailheader:message-id"" " & _
                        "FROM scope('shallow traversal of """ & strOfficeCalendarURL & """') " & _
                        "WHERE (""urn:schemas:mailheader:message-id"" = CAST(" & Chr(34) & strItem & Chr(34) & " as 'string')) "
           
                ' Open the recordset containing the one record to delete.
                .Open
            
                ' Move to the record to delete it.
                .MoveFirst
            
                ' Delete the record.
                .Delete
        
                ' Close the recordset.
                .Close
                
            End With
            
        End If
        
        ' Destroy the object variables.
        Set objPersonalAppointment = Nothing
        Set record = Nothing
        Set rst = Nothing

End Sub

Private Function GetInitials(strValue As String) As String
    
    Dim intPos As Integer
    Dim strInitials As String
    Dim strTemp As String
    Dim strName As String
    
    ' Parse out the Mailbox name from the URL string that was passed in.
    strTemp = strValue
    intPos = InStr(strTemp, "/MBX/")
    strTemp = Mid(strTemp, intPos + 5)
    intPos = InStr(strTemp, "/")
    strName = Mid(strTemp, 1, intPos - 1)
    intPos = InStr(strTemp, " ")
    
    If intPos = 0 Then
        ' There is only one name. This is the Administrators account.
        strInitials = "ADM- "
    Else
        ' Parse out the initials of the name.
        strInitials = UCase(Left(strName, 1) & Mid(strValue, intPos + 1, 1)) & "- " \ ""
    End If

    GetInitials = strInitials
    
End Function

Public Function GetExchangeURLDomainString() As String
   
   ' Get the domain name for the ExOLEDB URL string.
   ' Example: file://.backofficestorage/<domainname>/
      
   Dim objADSystemInfo As ActiveDs.ADSystemInfo
   Dim strDomainName As String
   
   ' Instantiate the ADSystemInfo object.
   Set objADSystemInfo = New ActiveDs.ADSystemInfo
   
   ' Get the domain name.
   strDomainName = objADSystemInfo.DomainDNSName
   
   ' If a domain name is found, build the domain URL string.
   If Len(strDomainName) <> 0 Then
   
      ' Build the EXOLEDB Domian URL string.
      GetExchangeURLDomainString = "file://./backofficestorage/" & strDomainName & "/"
   
   Else
   
      ' An error occurred
      Err.Raise vbObjectError + 9999, "Retrieval of domain name failed."
      
   End If
   
   ' Destroy object variable.
   Set objADSystemInfo = Nothing
   
End Function


